@@ -1,19 +1,14 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        menu.cpp
+// Name:        src/mac/carbon/menu.cpp
 // Purpose:     wxMenu, wxMenuBar, wxMenuItem
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     1998-01-01
-// RCS-ID:      $Id: menu.cpp,v 1.81 2005/08/08 21:51:32 VZ Exp $
+// RCS-ID:      $Id: menu.cpp 48053 2007-08-13 17:07:01Z JS $
 // Copyright:   (c) Stefan Csomor
-// Licence:       wxWindows licence
+// Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "menu.h"
-#pragma implementation "menuitem.h"
-#endif
-
 // ============================================================================
 // headers & declarations
 // ============================================================================
@@ -23,13 +18,15 @@
 
 #include "wx/wxprec.h"
 
-#include "wx/app.h"
 #include "wx/menu.h"
-#include "wx/menuitem.h"
-#include "wx/window.h"
-#include "wx/log.h"
-#include "wx/utils.h"
-#include "wx/frame.h"
+
+#ifndef WX_PRECOMP
+    #include "wx/log.h"
+    #include "wx/app.h"
+    #include "wx/utils.h"
+    #include "wx/frame.h"
+    #include "wx/menuitem.h"
+#endif
 
 #include "wx/mac/uma.h"
 
@@ -43,8 +40,7 @@ IMPLEMENT_DYNAMIC_CLASS(wxMenuBar, wxEvtHandler)
 // the (popup) menu title has this special id
 static const int idMenuTitle = -3;
 
-const short kwxMacMenuBarResource = 1 ;
-const short kwxMacAppleMenuId = 1 ;
+static const short kwxMacAppleMenuId = 1 ;
 
 
 // Find an item given the Macintosh Menu Reference
@@ -85,6 +81,54 @@ void wxRemoveMacMenuAssociation(wxMenu *menu)
     }
 }
 
+void wxInsertMenuItemsInMenu(wxMenu* menu, MenuRef wm, MenuItemIndex insertAfter)
+{
+    wxMenuItemList::compatibility_iterator node;
+    wxMenuItem *item;
+    wxMenu *subMenu = NULL ;
+    bool newItems = false;
+
+    for (node = menu->GetMenuItems().GetFirst(); node; node = node->GetNext())
+    {
+        item = (wxMenuItem *)node->GetData();
+        subMenu = item->GetSubMenu() ;
+        if (subMenu)
+        {
+            wxInsertMenuItemsInMenu(subMenu, (MenuRef)subMenu->GetHMenu(), 0);
+        }
+        if ( item->IsSeparator() )
+        {
+            if ( wm && newItems)
+                InsertMenuItemTextWithCFString( wm,
+                    CFSTR(""), insertAfter, kMenuItemAttrSeparator, 0);
+
+            newItems = false;
+        }
+        else
+        {
+            wxAcceleratorEntry*
+                entry = wxAcceleratorEntry::Create( item->GetText() ) ;
+
+            MenuItemIndex winListPos = (MenuItemIndex)-1;
+            OSStatus err = GetIndMenuItemWithCommandID(wm,
+                        wxIdToMacCommand ( item->GetId() ), 1, NULL, &winListPos);
+
+            if ( wm && err == menuItemNotFoundErr )
+            {
+                // NB: the only way to determine whether or not we should add
+                // a separator is to know if we've added menu items to the menu
+                // before the separator.
+                newItems = true;
+                UMAInsertMenuItem(wm, wxStripMenuCodes(item->GetText()) , wxFont::GetDefaultEncoding(), insertAfter, entry);
+                SetMenuItemCommandID( wm , insertAfter+1 , wxIdToMacCommand ( item->GetId() ) ) ;
+                SetMenuItemRefCon( wm , insertAfter+1 , (URefCon) item ) ;
+            }
+
+            delete entry ;
+        }
+    }
+}
+
 // ============================================================================
 // implementation
 // ============================================================================
@@ -107,7 +151,8 @@ _wxMenuAt(const wxMenuList &menuList, size_t pos)
 {
     wxMenuList::compatibility_iterator menuIter = menuList.GetFirst();
 
-    while (pos-- > 0) menuIter = menuIter->GetNext();
+    while (pos-- > 0)
+        menuIter = menuIter->GetNext();
 
     return menuIter->GetData() ;
 }
@@ -164,27 +209,28 @@ bool wxMenu::DoInsertOrAppend(wxMenuItem *pItem, size_t pos)
     if ( pItem->IsSeparator() )
     {
         if ( pos == (size_t)-1 )
-            MacAppendMenu(MAC_WXHMENU(m_hMenu), "\p-");
+            AppendMenuItemTextWithCFString( MAC_WXHMENU(m_hMenu),
+                CFSTR(""), kMenuItemAttrSeparator, 0,NULL);
         else
-            MacInsertMenuItem(MAC_WXHMENU(m_hMenu), "\p-" , pos);
+            InsertMenuItemTextWithCFString( MAC_WXHMENU(m_hMenu),
+                CFSTR(""), pos, kMenuItemAttrSeparator, 0);
     }
     else
     {
         wxMenu *pSubMenu = pItem->GetSubMenu() ;
         if ( pSubMenu != NULL )
         {
-               wxASSERT_MSG( pSubMenu->m_hMenu != NULL , wxT("invalid submenu added"));
+            wxASSERT_MSG( pSubMenu->m_hMenu != NULL , wxT("invalid submenu added"));
             pSubMenu->m_menuParent = this ;
 
             if (wxMenuBar::MacGetInstalledMenuBar() == GetMenuBar())
-            {
                 pSubMenu->MacBeforeDisplay( true ) ;
-             }
 
             if ( pos == (size_t)-1 )
-                UMAAppendSubMenuItem(MAC_WXHMENU(m_hMenu), pItem->GetText(), wxFont::GetDefaultEncoding() , pSubMenu->m_macMenuId);
+                UMAAppendSubMenuItem(MAC_WXHMENU(m_hMenu), wxStripMenuCodes(pItem->GetText()), wxFont::GetDefaultEncoding(), pSubMenu->m_macMenuId);
             else
-                UMAInsertSubMenuItem(MAC_WXHMENU(m_hMenu), pItem->GetText(), wxFont::GetDefaultEncoding()  , pos, pSubMenu->m_macMenuId);
+                UMAInsertSubMenuItem(MAC_WXHMENU(m_hMenu), wxStripMenuCodes(pItem->GetText()), wxFont::GetDefaultEncoding(), pos, pSubMenu->m_macMenuId);
+
             pItem->UpdateItemBitmap() ;
             pItem->UpdateItemStatus() ;
         }
@@ -204,23 +250,21 @@ bool wxMenu::DoInsertOrAppend(wxMenuItem *pItem, size_t pos)
                 pos += 1 ;
             }
 
-            SetMenuItemCommandID( MAC_WXHMENU(m_hMenu) , pos , pItem->GetId() ) ;
-            SetMenuItemRefCon( MAC_WXHMENU(m_hMenu) , pos , (UInt32) pItem ) ;
+            SetMenuItemCommandID( MAC_WXHMENU(m_hMenu) , pos , wxIdToMacCommand ( pItem->GetId() ) ) ;
+            SetMenuItemRefCon( MAC_WXHMENU(m_hMenu) , pos , (URefCon) pItem ) ;
             pItem->UpdateItemText() ;
             pItem->UpdateItemBitmap() ;
             pItem->UpdateItemStatus() ;
 
-              if ( pItem->GetId() == idMenuTitle )
-              {
+            if ( pItem->GetId() == idMenuTitle )
                 UMAEnableMenuItem(MAC_WXHMENU(m_hMenu) , pos , false ) ;
-              }
         }
     }
+
     // if we're already attached to the menubar, we must update it
     if ( IsAttached() && GetMenuBar()->IsAttached() )
-    {
         GetMenuBar()->Refresh();
-    }
+
     return true ;
 }
 
@@ -274,15 +318,11 @@ wxMenuItem* wxMenu::DoAppend(wxMenuItem *item)
     }
 
     if ( !wxMenuBase::DoAppend(item) || !DoInsertOrAppend(item) )
-    {
         return NULL;
-    }
 
     if ( check )
-    {
         // check the item initially
         item->Check(true);
-    }
 
     return item;
 }
@@ -291,8 +331,8 @@ wxMenuItem* wxMenu::DoInsert(size_t pos, wxMenuItem *item)
 {
     if (wxMenuBase::DoInsert(pos, item) && DoInsertOrAppend(item, pos))
         return item;
-    else
-        return NULL;
+
+    return NULL;
 }
 
 wxMenuItem *wxMenu::DoRemove(wxMenuItem *item)
@@ -300,6 +340,7 @@ wxMenuItem *wxMenu::DoRemove(wxMenuItem *item)
     // we need to find the items position in the child list
     size_t pos;
     wxMenuItemList::compatibility_iterator node = GetMenuItems().GetFirst();
+
     for ( pos = 0; node; pos++ )
     {
         if ( node->GetData() == item )
@@ -314,10 +355,8 @@ wxMenuItem *wxMenu::DoRemove(wxMenuItem *item)
     ::DeleteMenuItem(MAC_WXHMENU(m_hMenu) , pos + 1);
 
     if ( IsAttached() && GetMenuBar()->IsAttached() )
-    {
         // otherwise, the change won't be visible
         GetMenuBar()->Refresh();
-    }
 
     // and from internal data structures
     return wxMenuBase::DoRemove(item);
@@ -335,12 +374,10 @@ bool wxMenu::ProcessCommand(wxCommandEvent & event)
 
     // Try the menu's event handler
     if ( /* !processed && */ GetEventHandler())
-    {
         processed = GetEventHandler()->ProcessEvent(event);
-    }
 
-    // Try the window the menu was popped up from (and up through the
-    // hierarchy)
+    // Try the window the menu was popped up from
+    // (and up through the hierarchy)
     wxWindow *win = GetInvokingWindow();
     if ( !processed && win )
         processed = win->GetEventHandler()->ProcessEvent(event);
@@ -348,7 +385,6 @@ bool wxMenu::ProcessCommand(wxCommandEvent & event)
     return processed;
 }
 
-
 // ---------------------------------------------------------------------------
 // other
 // ---------------------------------------------------------------------------
@@ -410,14 +446,15 @@ void wxMenu::MacEnableMenu( bool bDoEnable )
 }
 
 // MacOS needs to know about submenus somewhere within this menu
-// before it can be displayed , also hide special menu items like preferences
-// that are handled by the OS
+// before it can be displayed, also hide special menu items
+// like preferences that are handled by the OS
 void wxMenu::MacBeforeDisplay( bool isSubMenu )
 {
     wxMenuItem* previousItem = NULL ;
     size_t pos ;
     wxMenuItemList::compatibility_iterator node;
     wxMenuItem *item;
+
     for (pos = 0, node = GetMenuItems().GetFirst(); node; node = node->GetNext(), pos++)
     {
         item = (wxMenuItem *)node->GetData();
@@ -428,7 +465,7 @@ void wxMenu::MacBeforeDisplay( bool isSubMenu )
         }
         else // normal item
         {
-            #if TARGET_CARBON
+#if TARGET_CARBON
             // what we do here is to hide the special items which are
             // shown in the application menu anyhow -- it doesn't make
             // sense to show them in their normal place as well
@@ -483,26 +520,26 @@ void wxMenu::MacBeforeDisplay( bool isSubMenu )
                                               0 );
                 }
             }
-            #endif // TARGET_CARBON
+#endif // TARGET_CARBON
         }
+
         previousItem = item ;
     }
 
     if ( isSubMenu )
         ::InsertMenu(MAC_WXHMENU( GetHMenu()), -1);
-
 }
+
 // undo all changes from the MacBeforeDisplay call
 void wxMenu::MacAfterDisplay( bool isSubMenu )
 {
     if ( isSubMenu )
         ::DeleteMenu(MacGetMenuId());
 
-    wxMenuItem* previousItem = NULL ;
-    int pos ;
     wxMenuItemList::compatibility_iterator node;
     wxMenuItem *item;
-    for (pos = 0, node = GetMenuItems().GetFirst(); node; node = node->GetNext(), pos++)
+
+    for (node = GetMenuItems().GetFirst(); node; node = node->GetNext())
     {
         item = (wxMenuItem *)node->GetData();
         wxMenu* subMenu = item->GetSubMenu() ;
@@ -514,7 +551,6 @@ void wxMenu::MacAfterDisplay( bool isSubMenu )
         {
             // no need to undo hidings
         }
-        previousItem = item ;
     }
 }
 
@@ -559,7 +595,6 @@ wxMenuBar::wxMenuBar( long WXUNUSED(style) )
     Init();
 }
 
-
 wxMenuBar::wxMenuBar(size_t count, wxMenu *menus[], const wxString titles[], long WXUNUSED(style))
 {
     Init();
@@ -579,12 +614,12 @@ wxMenuBar::~wxMenuBar()
 {
     if (s_macCommonMenuBar == this)
         s_macCommonMenuBar = NULL;
+
     if (s_macInstalledMenuBar == this)
     {
         ::ClearMenuBar();
         s_macInstalledMenuBar = NULL;
     }
-
 }
 
 void wxMenuBar::Refresh(bool WXUNUSED(eraseBackground), const wxRect *WXUNUSED(rect))
@@ -600,28 +635,30 @@ void wxMenuBar::MacInstallMenuBar()
         return ;
 
     MenuBarHandle menubar = NULL ;
+
 #if TARGET_API_MAC_OSX
     menubar = NewHandleClear( 6 /* sizeof( MenuBarHeader ) */ ) ;
 #else
     menubar = NewHandleClear( 12 ) ;
     (*menubar)[3] = 0x0a ;
 #endif
+
     ::SetMenuBar( menubar ) ;
     DisposeMenuBar( menubar ) ;
     MenuHandle appleMenu = NULL ;
-    char appleMenuTitle[3] = { 01 , kMenuAppleLogoFilledGlyph , 0 } ;
 
     verify_noerr( CreateNewMenu( kwxMacAppleMenuId , 0 , &appleMenu ) ) ;
-    verify_noerr( SetMenuTitle( appleMenu , (ConstStr255Param) appleMenuTitle ) );
+    verify_noerr( SetMenuTitleWithCFString( appleMenu , CFSTR( "\x14" ) ) );
 
     // Add About/Preferences separator only on OS X
     // KH/RN: Separator is always present on 10.3 but not on 10.2
     // However, the change from 10.2 to 10.3 suggests it is preferred
 #if TARGET_API_MAC_OSX
-    MacInsertMenuItem( appleMenu , "\p-" , 0 ) ;
+    InsertMenuItemTextWithCFString( appleMenu,
+                CFSTR(""), 0, kMenuItemAttrSeparator, 0);
 #endif
-
-    MacInsertMenuItem( appleMenu , "\pAbout..." , 0 ) ;
+    InsertMenuItemTextWithCFString( appleMenu,
+                CFSTR("About..."), 0, 0, 0);
     MacInsertMenu( appleMenu , 0 ) ;
 
     // clean-up the help menu before adding new items
@@ -633,15 +670,14 @@ void wxMenuBar::MacInstallMenuBar()
         if ( UMAGetHelpMenu( &mh , &firstUserHelpMenuItem) == noErr )
         {
             for ( int i = CountMenuItems( mh ) ; i >= firstUserHelpMenuItem ; --i )
-            {
                 DeleteMenuItem( mh , i ) ;
-            }
         }
         else
         {
             mh = NULL ;
         }
     }
+
 #if TARGET_CARBON
     if ( UMAGetSystemVersion() >= 0x1000 && wxApp::s_macPreferencesMenuItemId)
     {
@@ -651,6 +687,7 @@ void wxMenuBar::MacInstallMenuBar()
         else
             EnableMenuCommand( NULL , kHICommandPreferences ) ;
     }
+
     // Unlike preferences which may or may not exist, the Quit item should be always
     // enabled unless it is added by the application and then disabled, otherwise
     // a program would be required to add an item with wxID_EXIT in order to get the
@@ -664,18 +701,17 @@ void wxMenuBar::MacInstallMenuBar()
             EnableMenuCommand( NULL , kHICommandQuit ) ;
     }
 #endif
+
     wxMenuList::compatibility_iterator menuIter = m_menus.GetFirst();
-    //
     for (size_t i = 0; i < m_menus.GetCount(); i++, menuIter = menuIter->GetNext())
     {
         wxMenuItemList::compatibility_iterator node;
         wxMenuItem *item;
-        int pos ;
         wxMenu* menu = menuIter->GetData() , *subMenu = NULL ;
 
-        if( m_titles[i] == wxT("?") || m_titles[i] == wxT("&?")  || m_titles[i] == wxApp::s_macHelpMenuTitleName )
+        if ( m_titles[i] == wxT("?") || m_titles[i] == wxT("&?")  || m_titles[i] == wxApp::s_macHelpMenuTitleName )
         {
-            for (pos = 0 , node = menu->GetMenuItems().GetFirst(); node; node = node->GetNext(), pos++)
+            for (node = menu->GetMenuItems().GetFirst(); node; node = node->GetNext())
             {
                 item = (wxMenuItem *)node->GetData();
                 subMenu = item->GetSubMenu() ;
@@ -690,24 +726,24 @@ void wxMenuBar::MacInstallMenuBar()
                         if ( mh == NULL )
                         {
                             MenuItemIndex firstUserHelpMenuItem ;
-                            if ( UMAGetHelpMenu( &mh , &firstUserHelpMenuItem) == noErr )
-                            {
-                            }
-                            else
+                            if ( UMAGetHelpMenu( &mh , &firstUserHelpMenuItem) != noErr )
                             {
                                 mh = NULL ;
                                 break ;
                             }
                         }
                     }
+
                     if ( item->IsSeparator() )
                     {
                         if ( mh )
-                            MacAppendMenu(mh, "\p-" );
+                            AppendMenuItemTextWithCFString( mh,
+                                CFSTR(""), kMenuItemAttrSeparator, 0,NULL);
                     }
                     else
                     {
-                        wxAcceleratorEntry* entry = wxGetAccelFromString( item->GetText() ) ;
+                        wxAcceleratorEntry*
+                            entry = wxAcceleratorEntry::Create( item->GetText() ) ;
 
                         if ( item->GetId() == wxApp::s_macAboutMenuItemId )
                         {
@@ -717,9 +753,11 @@ void wxMenuBar::MacInstallMenuBar()
                         {
                             if ( mh )
                             {
-                                UMAAppendMenuItem(mh, item->GetText()  , wxFont::GetDefaultEncoding(), entry);
-                                SetMenuItemCommandID( mh , CountMenuItems(mh) , item->GetId() ) ;
-                                SetMenuItemRefCon( mh , CountMenuItems(mh) , (UInt32)item ) ;
+                                UMAAppendMenuItem(mh, wxStripMenuCodes(item->GetText()) , wxFont::GetDefaultEncoding(), entry);
+                                MenuItemIndex position = CountMenuItems(mh);
+                                SetMenuItemCommandID( mh , position, wxIdToMacCommand ( item->GetId() ) );
+                                SetMenuItemRefCon( mh , position, (URefCon) item );
+                                item->DoUpdateItemBitmap( mh, position );
                             }
                         }
 
@@ -728,35 +766,70 @@ void wxMenuBar::MacInstallMenuBar()
                 }
             }
         }
+
+        else if ( ( m_titles[i] == wxT("Window") || m_titles[i] == wxT("&Window") )
+                && GetAutoWindowMenu() )
+        {
+            if ( MacGetWindowMenuHMenu() == NULL )
+            {
+                CreateStandardWindowMenu( 0 , (MenuHandle*) &s_macWindowMenuHandle ) ;
+            }
+
+            MenuRef wm = (MenuRef)MacGetWindowMenuHMenu();
+            if ( wm == NULL )
+                break;
+
+            // get the insertion point in the standard menu
+            MenuItemIndex winListStart;
+            GetIndMenuItemWithCommandID(wm,
+                        kHICommandWindowListSeparator, 1, NULL, &winListStart);
+
+            // add a separator so that the standard items and the custom items
+            // aren't mixed together, but only if this is the first run
+            OSStatus err = GetIndMenuItemWithCommandID(wm,
+                        'WXWM', 1, NULL, NULL);
+
+            if ( err == menuItemNotFoundErr )
+            {
+                InsertMenuItemTextWithCFString( wm,
+                        CFSTR(""), winListStart-1, kMenuItemAttrSeparator, 'WXWM');
+            }
+
+            wxInsertMenuItemsInMenu(menu, wm, winListStart);
+        }
         else
         {
             UMASetMenuTitle( MAC_WXHMENU(menu->GetHMenu()) , m_titles[i], m_font.GetEncoding()  ) ;
             menu->MacBeforeDisplay(false) ;
+
             ::InsertMenu(MAC_WXHMENU(_wxMenuAt(m_menus, i)->GetHMenu()), 0);
         }
     }
+
     // take care of the about menu item wherever it is
     {
         wxMenu* aboutMenu ;
         wxMenuItem *aboutMenuItem = FindItem(wxApp::s_macAboutMenuItemId , &aboutMenu) ;
         if ( aboutMenuItem )
         {
-            wxAcceleratorEntry* entry = wxGetAccelFromString( aboutMenuItem->GetText() ) ;
-            UMASetMenuItemText( GetMenuHandle( kwxMacAppleMenuId ) , 1 , aboutMenuItem->GetText() , wxFont::GetDefaultEncoding() );
+            wxAcceleratorEntry*
+                entry = wxAcceleratorEntry::Create( aboutMenuItem->GetText() ) ;
+            UMASetMenuItemText( GetMenuHandle( kwxMacAppleMenuId ) , 1 , wxStripMenuCodes ( aboutMenuItem->GetText() ) , wxFont::GetDefaultEncoding() );
             UMAEnableMenuItem( GetMenuHandle( kwxMacAppleMenuId ) , 1 , true );
-            SetMenuItemCommandID( GetMenuHandle( kwxMacAppleMenuId ) , 1 , aboutMenuItem->GetId() ) ;
-            SetMenuItemRefCon(GetMenuHandle( kwxMacAppleMenuId ) , 1 , (UInt32)aboutMenuItem ) ;
+            SetMenuItemCommandID( GetMenuHandle( kwxMacAppleMenuId ) , 1 , kHICommandAbout ) ;
+            SetMenuItemRefCon(GetMenuHandle( kwxMacAppleMenuId ) , 1 , (URefCon)aboutMenuItem ) ;
             UMASetMenuItemShortcut( GetMenuHandle( kwxMacAppleMenuId ) , 1 , entry ) ;
         }
     }
+
     if ( GetAutoWindowMenu() )
     {
         if ( MacGetWindowMenuHMenu() == NULL )
-        {
             CreateStandardWindowMenu( 0 , (MenuHandle*) &s_macWindowMenuHandle ) ;
-        }
+
         InsertMenu( (MenuHandle) MacGetWindowMenuHMenu() , 0 ) ;
     }
+
     ::DrawMenuBar() ;
     s_macInstalledMenuBar = this;
 }
@@ -764,18 +837,19 @@ void wxMenuBar::MacInstallMenuBar()
 void wxMenuBar::EnableTop(size_t pos, bool enable)
 {
     wxCHECK_RET( IsAttached(), wxT("doesn't work with unattached menubars") );
+
     _wxMenuAt(m_menus, pos)->MacEnableMenu( enable ) ;
     Refresh();
 }
 
-bool wxMenuBar::Enable( bool enable)
+bool wxMenuBar::Enable(bool enable)
 {
     wxCHECK_MSG( IsAttached(), false, wxT("doesn't work with unattached menubars") );
+
     size_t i;
     for (i = 0; i < GetMenuCount(); i++)
-    {
         EnableTop(i, enable);
-    }
+
     return true;
 }
 
@@ -786,9 +860,7 @@ void wxMenuBar::SetLabelTop(size_t pos, const wxString& label)
     m_titles[pos] = label;
 
     if ( !IsAttached() )
-    {
         return;
-    }
 
     _wxMenuAt(m_menus, pos)->SetTitle( label ) ;
 
@@ -804,6 +876,15 @@ wxString wxMenuBar::GetLabelTop(size_t pos) const
     wxCHECK_MSG( pos < GetMenuCount(), wxEmptyString,
                  wxT("invalid menu index in wxMenuBar::GetLabelTop") );
 
+    return wxStripMenuCodes(m_titles[pos]);
+}
+
+// Gets the original label at the top-level of the menubar
+wxString wxMenuBar::GetMenuLabel(size_t pos) const
+{
+    wxCHECK_MSG( pos < GetMenuCount(), wxEmptyString,
+                 wxT("invalid menu index in wxMenuBar::GetMenuLabel") );
+
     return m_titles[pos];
 }
 
@@ -820,10 +901,8 @@ int wxMenuBar::FindMenu(const wxString& title)
     }
 
     return wxNOT_FOUND;
-
 }
 
-
 // ---------------------------------------------------------------------------
 // wxMenuBar construction
 // ---------------------------------------------------------------------------
@@ -832,7 +911,8 @@ wxMenu *wxMenuBar::Replace(size_t pos, wxMenu *menu, const wxString& title)
 {
     wxMenu *menuOld = wxMenuBarBase::Replace(pos, menu, title);
     if ( !menuOld )
-        return false;
+        return NULL;
+
     m_titles[pos] = title;
 
     if ( IsAttached() )
@@ -841,22 +921,18 @@ wxMenu *wxMenuBar::Replace(size_t pos, wxMenu *menu, const wxString& title)
         {
             menuOld->MacAfterDisplay( false ) ;
             ::DeleteMenu( menuOld->MacGetMenuId() /* m_menus[pos]->MacGetMenuId() */ ) ;
-            {
-                menu->MacBeforeDisplay( false ) ;
-                UMASetMenuTitle( MAC_WXHMENU(menu->GetHMenu()) , title , m_font.GetEncoding() ) ;
-                if ( pos == m_menus.GetCount() - 1)
-                {
-                    ::InsertMenu( MAC_WXHMENU(menu->GetHMenu()) , 0 ) ;
-                }
-                else
-                {
-                    ::InsertMenu( MAC_WXHMENU(menu->GetHMenu()) , _wxMenuAt(m_menus, pos+1)->MacGetMenuId() ) ;
-                }
-            }
+
+            menu->MacBeforeDisplay( false ) ;
+            UMASetMenuTitle( MAC_WXHMENU(menu->GetHMenu()) , title , m_font.GetEncoding() ) ;
+            if ( pos == m_menus.GetCount() - 1)
+                ::InsertMenu( MAC_WXHMENU(menu->GetHMenu()) , 0 ) ;
+            else
+                ::InsertMenu( MAC_WXHMENU(menu->GetHMenu()) , _wxMenuAt(m_menus, pos + 1)->MacGetMenuId() ) ;
         }
 
         Refresh();
     }
+
     if (m_invokingWindow)
         wxMenubarSetInvokingWindow( menu, m_invokingWindow );
 
@@ -877,17 +953,16 @@ bool wxMenuBar::Insert(size_t pos, wxMenu *menu, const wxString& title)
         if (s_macInstalledMenuBar == this)
         {
             menu->MacBeforeDisplay( false ) ;
+
             if ( pos == (size_t) -1  || pos + 1 == m_menus.GetCount() )
-            {
                 ::InsertMenu( MAC_WXHMENU(menu->GetHMenu()) , 0 ) ;
-            }
             else
-            {
                 ::InsertMenu( MAC_WXHMENU(menu->GetHMenu()) , _wxMenuAt(m_menus, pos+1)->MacGetMenuId() ) ;
-            }
         }
+
         Refresh();
     }
+
     if (m_invokingWindow)
         wxMenubarSetInvokingWindow( menu, m_invokingWindow );
 
@@ -903,9 +978,7 @@ wxMenu *wxMenuBar::Remove(size_t pos)
     if ( IsAttached() )
     {
         if (s_macInstalledMenuBar == this)
-        {
             ::DeleteMenu( menu->MacGetMenuId() /* m_menus[pos]->MacGetMenuId() */ ) ;
-        }
 
         Refresh();
     }
@@ -918,7 +991,7 @@ wxMenu *wxMenuBar::Remove(size_t pos)
 bool wxMenuBar::Append(wxMenu *menu, const wxString& title)
 {
     WXHMENU submenu = menu ? menu->GetHMenu() : 0;
-    wxCHECK_MSG( submenu, false, wxT("can't append invalid menu to menubar") );
+        wxCHECK_MSG( submenu, false, wxT("can't append invalid menu to menubar") );
 
     if ( !wxMenuBarBase::Append(menu, title) )
         return false;
@@ -949,13 +1022,14 @@ bool wxMenuBar::Append(wxMenu *menu, const wxString& title)
 static void wxMenubarUnsetInvokingWindow( wxMenu *menu )
 {
     menu->SetInvokingWindow( (wxWindow*) NULL );
-
     wxMenuItemList::compatibility_iterator node = menu->GetMenuItems().GetFirst();
+
     while (node)
     {
         wxMenuItem *menuitem = node->GetData();
         if (menuitem->IsSubMenu())
             wxMenubarUnsetInvokingWindow( menuitem->GetSubMenu() );
+
         node = node->GetNext();
     }
 }
@@ -963,13 +1037,15 @@ static void wxMenubarUnsetInvokingWindow( wxMenu *menu )
 static void wxMenubarSetInvokingWindow( wxMenu *menu, wxWindow *win )
 {
     menu->SetInvokingWindow( win );
-
+    wxMenuItem *menuitem;
     wxMenuItemList::compatibility_iterator node = menu->GetMenuItems().GetFirst();
+
     while (node)
     {
-        wxMenuItem *menuitem = node->GetData();
+        menuitem = node->GetData();
         if (menuitem->IsSubMenu())
             wxMenubarSetInvokingWindow( menuitem->GetSubMenu() , win );
+
         node = node->GetNext();
     }
 }
@@ -977,11 +1053,14 @@ static void wxMenubarSetInvokingWindow( wxMenu *menu, wxWindow *win )
 void wxMenuBar::UnsetInvokingWindow()
 {
     m_invokingWindow = (wxWindow*) NULL;
+    wxMenu *menu;
     wxMenuList::compatibility_iterator node = m_menus.GetFirst();
+
     while (node)
     {
-        wxMenu *menu = node->GetData();
+        menu = node->GetData();
         wxMenubarUnsetInvokingWindow( menu );
+
         node = node->GetNext();
     }
 }
@@ -989,11 +1068,14 @@ void wxMenuBar::UnsetInvokingWindow()
 void wxMenuBar::SetInvokingWindow(wxFrame *frame)
 {
     m_invokingWindow = frame;
+    wxMenu *menu;
     wxMenuList::compatibility_iterator node = m_menus.GetFirst();
+
     while (node)
     {
-        wxMenu *menu = node->GetData();
+        menu = node->GetData();
         wxMenubarSetInvokingWindow( menu, frame );
+
         node = node->GetNext();
     }
 }
@@ -1007,6 +1089,7 @@ void wxMenuBar::Attach(wxFrame *frame)
 {
     wxMenuBarBase::Attach( frame ) ;
 }
+
 // ---------------------------------------------------------------------------
 // wxMenuBar searching for menu items
 // ---------------------------------------------------------------------------
@@ -1035,11 +1118,7 @@ wxMenuItem *wxMenuBar::FindItem(int id, wxMenu **itemMenu) const
     wxMenuItem *item = NULL;
     size_t count = GetMenuCount();
     for ( size_t i = 0; !item && (i < count); i++ )
-    {
         item = _wxMenuAt(m_menus, i)->FindItem(id, itemMenu);
-    }
 
     return item;
 }
-
-
