@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     17/09/98
-// RCS-ID:      $Id: window.cpp,v 1.163 2006/10/15 15:51:35 MBN Exp $
+// RCS-ID:      $Id: window.cpp 48765 2007-09-18 14:15:27Z JS $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -641,7 +641,7 @@ void wxWindow::SetLabel(const wxString& label)
 
 wxString wxWindow::GetLabel() const
 {
-    char *label;
+    char *label = NULL;
     XtVaGetValues((Widget)GetMainWidget(), XmNtitle, &label, NULL);
 
     return wxString(label);
@@ -1644,7 +1644,7 @@ void wxWindow::OnInternalIdle()
 {
     // This calls the UI-update mechanism (querying windows for
     // menu/toolbar/control state information)
-    if (wxUpdateUIEvent::CanUpdate(this))
+    if (wxUpdateUIEvent::CanUpdate(this) && IsShown())
         UpdateWindowUI(wxUPDATE_UI_FROMIDLE);
 }
 
